'use strict';

var server = require('server');
var RedsysConstants = require('*/cartridge/scripts/constants/redsysConstants');
var Logger = require('dw/system/Logger');
var logger = Logger.getLogger('Redsys', 'Redsys');

/**
 * Redsys-Form : This endpoint is invoked to send order information to Redsys Virtual TPV
 * @param {middleware} server.middleware.https - server.middleware.https
 * @param {httpparameter} orderID - Order Id.
 * @param {httpparameter} orderToken - Order Token.
 * @param {serverfunction} post
 * @param {renders} - isml
 */
server.post(
    'Form',
    server.middleware.https,
    function (req, res, next) {
        var CustomObjectMgr = require('dw/object/CustomObjectMgr');
        var OrderMgr = require('dw/order/OrderMgr');
        var RedsysHelper = require('*/cartridge/scripts/helpers/redsysHelper.js');
        var Resource = require('dw/web/Resource');

        if (!req.form.orderToken || !req.form.orderID) {
            res.render('/error', {
                message: Resource.msg('error.confirmation.error', 'confirmation', null)
            });

            next();
        }

        var paymentMethodId = req.session.privacyCache.get('redsys_paymentMethod');
        var redirectionUrl = null;

        if (paymentMethodId === 'REDSYS_CREDIT_CARD') {
            redirectionUrl = RedsysHelper.getRedsysCreditCardRedirectionUrl();
        }

        if (paymentMethodId === 'REDSYS_BIZUM') {
            redirectionUrl = RedsysHelper.getRedsysBizumRedirectionUrl();
        }

        var order = OrderMgr.getOrder(req.form.orderID, req.form.orderToken);
        var RedsysPaymentForm = CustomObjectMgr.getCustomObject('RedsysNotifications', order.orderNo);

        var redsysFormData = {
            dsSignatureVersion: RedsysConstants.RESDSYS_DS_SIGNATURE_VERSION,
            dsSignature: RedsysPaymentForm.custom.ds_signature,
            dsMerchantParameters: RedsysPaymentForm.custom.ds_merchant_parameters,
            redirectionUrl: redirectionUrl
        };

        res.render('checkout/billing/paymentOptions/redsys/redirectionForm', {
            redsysFormData: redsysFormData
        });

        next();
    });

/**
 * Redsys-Data : This endpoint is invoked to send order information to Redsys Virtual TPV in SFRA versions early to 6.0.0
 * @param {middleware} server.middleware.https - server.middleware.https
 * @param {querystringparameter} order - Order ID
 * @param {querystringparameter} orderToken - token associated with the order
 * @param {serverfunction} - get
 * @param {renders} - isml
 */
server.get(
    'Data',
    server.middleware.https,
    function (req, res, next) {
        var CustomObjectMgr = require('dw/object/CustomObjectMgr');
        var OrderMgr = require('dw/order/OrderMgr');
        var RedsysHelper = require('*/cartridge/scripts/helpers/redsysHelper.js');
        var Resource = require('dw/web/Resource');

        var order = OrderMgr.getOrder(req.querystring.ID);
        var orderToken = req.querystring.token ? req.querystring.token : null;

        if (!orderToken || !order) {
            res.render('/error', {
                message: Resource.msg('error.confirmation.error', 'confirmation', null)
            });

            next();
        }

        var paymentMethodId = req.session.privacyCache.get('redsys_paymentMethod');
        var redirectionUrl = null;

        if (paymentMethodId === 'REDSYS_CREDIT_CARD') {
            redirectionUrl = RedsysHelper.getRedsysCreditCardRedirectionUrl();
        }

        if (paymentMethodId === 'REDSYS_BIZUM') {
            redirectionUrl = RedsysHelper.getRedsysBizumRedirectionUrl();
        }

        var currentOrder = OrderMgr.getOrder(order.orderNo, orderToken);
        var RedsysPaymentForm = CustomObjectMgr.getCustomObject('RedsysNotifications', currentOrder.orderNo);

        var redsysFormData = {
            dsSignatureVersion: RedsysConstants.RESDSYS_DS_SIGNATURE_VERSION,
            dsSignature: RedsysPaymentForm.custom.ds_signature,
            dsMerchantParameters: RedsysPaymentForm.custom.ds_merchant_parameters,
            redirectionUrl: redirectionUrl
        };

        res.render('checkout/billing/paymentOptions/redsys/redirectionForm', {
            redsysFormData: redsysFormData
        });

        next();
    });

/**
 * Redsys-OK : This endpoint is invoked to receive Redsys response after payment process with result success
 * @param {middleware} server.middleware.https - server.middleware.https
 * @param {querystringparameter} orderID - Order ID
 * @param {serverfunction} - get
 * @param {renders} - isml
 */
server.get(
    'OK',
    server.middleware.https,
    function (req, res, next) {
        var COHelpers = require('*/cartridge/scripts/checkout/checkoutHelpers');
        var Order = require('dw/order/Order');
        var OrderMgr = require('dw/order/OrderMgr');

        var orderID = req.querystring.orderID;
        var order = OrderMgr.searchOrders('orderNo={0}', 'creationDate desc', orderID).first();

        if (!order) {
            res.setStatusCode(500);
            logger.error('Redsys-OK: order {0} not found', orderID);
            return next();
        }

        // Sends a confirmation to the current user when Redsys approved payment and the order status is not failed
        if (order.status.value !== Order.ORDER_STATUS_FAILED) {
            COHelpers.sendConfirmationEmail(order, req.locale.id);
        }

        res.render('checkout/redsys/redsysConfirmOrder', {
            orderID: order.orderNo,
            orderToken: order.orderToken
        });

        next();
    });

/**
 * Redsys-KO : This endpoint is invoked to receive Redsys response after payment process with result error
 * @param {middleware} server.middleware.https - server.middleware.https
 * @param {querystringparameter} orderID - Order ID
 * @param {serverfunction} - get
* @return {dw.web.Url} THE url to redirect to in case of error with payment.
 */
server.get('KO',
    server.middleware.https,
    function (req, res, next) {
        var OrderMgr = require('dw/order/OrderMgr');
        var Transaction = require('dw/system/Transaction');
        var URLUtils = require('dw/web/URLUtils');

        var orderID = req.querystring.orderID;
        var order = OrderMgr.searchOrders('orderNo={0}', 'creationDate desc', orderID).first();

        if(!order) {
            logger.error('Redsys-KO: Order {0} not found', orderID);
            res.redirect(URLUtils.url('Checkout-Begin', 'stage', 'payment'));
        } else {
            try {
                Transaction.wrap(function () {
                    OrderMgr.failOrder(order, true);
                });
            } catch (err) {
                logger.error('Redsys-KO: Order {0} not found or was an error updating order status: {1}', orderID, err);
            }
        }

        res.redirect(URLUtils.url('Checkout-Begin', 'stage', 'payment'));

        next();
    });

/**
 * Redsys-Notify : This endpoint is invoked by Redsys to update order information
 * @param {middleware} server.middleware.https - server.middleware.https
 * @param {httpparameter} parameters - Redsys response params.
 * @param {serverfunction} - post
 * @param {returns} - json
 */
server.post('Notify',
    server.middleware.https,
    function (req, res, next) {
        var CustomObjectMgr = require('dw/object/CustomObjectMgr');
        var Transaction = require('dw/system/Transaction');
        var StringUtils = require('dw/util/StringUtils');
        var Status = require('dw/system/Status');
        var RedsysHelper = require('*/cartridge/scripts/helpers/redsysHelper.js');
        var Order = require('dw/order/Order');
        var OrderMgr = require('dw/order/OrderMgr');

        var parameters = JSON.parse(StringUtils.decodeBase64(req.form.Ds_MerchantParameters));
        var regexpbar = /\//g;
        var regexpplus = /\+/g;

        var redsysNotify = CustomObjectMgr.getCustomObject('RedsysNotifications', parameters.Ds_Order);
        if (redsysNotify) {
            var order = OrderMgr.getOrder(redsysNotify.custom.redsysNotifyId, redsysNotify.custom.orderToken);
            var paymentInstruments = order.getPaymentInstruments();
            var paymentMethod = paymentInstruments[0].paymentMethod;

            var ourSignature = RedsysHelper.makeParameters(paymentMethod, parameters, parameters.Ds_Order).Ds_Signature.replace(regexpbar, '_').replace(regexpplus, '-');
            var dsResponse = parameters.Ds_Response;
            var dsSignature = req.form.Ds_Signature;

            // Check that the redsys data is correct and the signature is the same.
            if (ourSignature === dsSignature && dsResponse > -1 && dsResponse < 100) {
                try {
                    Transaction.begin();
                    var placeOrderStatus = OrderMgr.placeOrder(order);
                    if (placeOrderStatus === Status.ERROR) throw new Error();
                    order.setConfirmationStatus(Order.CONFIRMATION_STATUS_CONFIRMED);
                    order.setExportStatus(Order.EXPORT_STATUS_READY);
                    order.setPaymentStatus(Order.PAYMENT_STATUS_PAID);
                    redsysNotify.custom.ds_merchant_parameters = JSON.stringify(parameters);
                    Transaction.commit();

                    Transaction.wrap(function () {
                        if (paymentMethod === 'REDSYS_CREDIT_CARD') {
                            paymentInstruments[0].custom.redsysCreditCardPaymentID = parameters.Ds_Order;
                            paymentInstruments[0].custom.redsysCreditCardPaymentData = JSON.stringify(parameters);
                        }

                        if (paymentMethod === 'REDSYS_BIZUM') {
                            paymentInstruments[0].custom.redsysBizumPaymentID = parameters.Ds_Order;
                            paymentInstruments[0].custom.redsysBizumPaymentData = JSON.stringify(parameters);
                        }
                    });
                } catch (err) {
                    logger.error('Error updating order: {0} - {1}', parameters.Ds_Order, err);
                    res.setStatusCode(500);
                    res.json({
                        error: true,
                        msg: 'Error updating order'
                    });
                }
            }
        } else {
            logger.error('RedsysNotification not found with order number {0}. Order was not updated to status paid', parameters.Ds_Order);
            res.setStatusCode(500);
            res.json({
                error: true,
                msg: 'Error updating order'
            });
        }

        res.setStatusCode(200);
        res.json({
            error: true,
            msg: 'Order updated'
        });

        next();
    });

module.exports = server.exports();
