/* eslint-disable no-shadow */
/* eslint-disable no-unused-vars */

'use strict';

var Site = require('dw/system/Site');
var RedsysConstants = require('*/cartridge/scripts/constants/redsysConstants');
var Logger = require('dw/system/Logger');
var logger = Logger.getLogger('Redsys', 'Redsys');
// eslint-disable-next-line no-use-before-define
const redsysCreditCardPreferences = getRedsysCreditCardPreferences();
// eslint-disable-next-line no-use-before-define
const redsysBizumPreferences = getRedsysBizumPreferences();

/**
 * Checks if payment with Redsys - Credit card is enabled
 * @returns {boolean} Is payment enabled
 */
function isRedsysCreditCardEnabled() {
    var PaymentMgr = require('dw/order/PaymentMgr');
    var activePaymentMethods = PaymentMgr.getActivePaymentMethods();
    var redsysCreditCard = 'REDSYS_CREDIT_CARD';
    var isEnabled = false;

    // eslint-disable-next-line consistent-return
    Array.some(activePaymentMethods, function (paymentMethod) {
        if (paymentMethod.paymentProcessor.ID === redsysCreditCard) {
            isEnabled = true;
            return true;
        }
    });

    return isEnabled;
}

/**
 * Checks if payment with Redsys - Bizum is enabled
 * @returns {boolean} Is payment enabled
 */
function isRedsysBizumEnabled() {
    var PaymentMgr = require('dw/order/PaymentMgr');
    var activePaymentMethods = PaymentMgr.getActivePaymentMethods();
    var redsysBizum = 'REDSYS_BIZUM';
    var isEnabled = false;

    // eslint-disable-next-line consistent-return
    Array.some(activePaymentMethods, function (paymentMethod) {
        if (paymentMethod.paymentProcessor.ID === redsysBizum) {
            isEnabled = true;
            return true;
        }
    });

    return isEnabled;
}

/**
 * Bizum redirection url
 * @returns {string} Redirection url
 */
function getRedsysBizumRedirectionUrl() {
    return redsysBizumPreferences.redirectionUrl.value;
}

/**
 * Credit card redirection url
 * @returns {string} Redirection url
 */
function getRedsysCreditCardRedirectionUrl() {
    return redsysCreditCardPreferences.redirectionUrl.value;
}

/**
 * Repeat character.
 * @param {string} char - character
 * @param {int} length - length
 * @returns {string} character / space
 */
function repeatCharacter(char, length) {
    if (length > 0) {
        return char.repeat(length);
    }

    return '';
}

/**
 * Get Bizum custom preferences
 * @returns {Object} custom preferences
 */
function getRedsysBizumPreferences() {
    var site = Site.current;
    var redsysBizumPreferences = {
        merchantCode: site.getCustomPreferenceValue('redsys_bizum_merchant_code'),
        merchantSecret: site.getCustomPreferenceValue('redsys_bizum_merchant_secret'),
        merchantTerminal: site.getCustomPreferenceValue('redsys_bizum_merchant_terminal'),
        redirectionUrl: site.getCustomPreferenceValue('redsys_bizum_redirection_url'),
        limitTotal: site.getCustomPreferenceValue('redsys_bizum_limit_total')
    };

    return redsysBizumPreferences;
}

/**
 * Get Redsys preferences
 * @returns {Object} custom preferences
 */
function getRedsysCreditCardPreferences() {
    var site = Site.current;
    var redsysCreditCardPreferences = {
        merchantCode: site.getCustomPreferenceValue('redsys_credit_card_merchant_code'),
        merchantSecret: site.getCustomPreferenceValue('redsys_credit_card_merchant_secret'),
        merchantTerminal: site.getCustomPreferenceValue('redsys_credit_card_merchant_terminal'),
        redirectionUrl: site.getCustomPreferenceValue('redsys_credit_card_redirection_url')
    };

    return redsysCreditCardPreferences;
}

/**
 * Get Required configuration values to pay with Redsys using Bizum
 * @param {string} OrderID - Customer's order ID
 * @param {string} ammount - Amount to pay
 * @returns {Object} - Required config to pay with Bizum
 */
function getBizumConfig(OrderID, ammount) {
    var URLUtils = require('dw/web/URLUtils');

    var obj = {};
    // eslint-disable-next-line no-undef
    const currencyCode = session.currency.currencyCode;
    if (currencyCode !== 'EUR') {
        obj = {
            error: true,
            message: 'currency error'
        };

        return obj;
    }

    obj = {
        DS_MERCHANT_AMOUNT: ammount * 100,
        DS_MERCHANT_CURRENCY: '978',
        DS_MERCHANT_MERCHANTCODE: redsysBizumPreferences.merchantCode,
        DS_MERCHANT_MERCHANTURL: URLUtils.https('Redsys-Notify').toString(),
        DS_MERCHANT_ORDER: OrderID,
        DS_MERCHANT_TERMINAL: redsysBizumPreferences.merchantTerminal,
        DS_MERCHANT_TRANSACTIONTYPE: '0',
        DS_MERCHANT_URLKO: URLUtils.https('Redsys-KO', 'orderID', OrderID).toString(),
        DS_MERCHANT_URLOK: URLUtils.https('Redsys-OK', 'orderID', OrderID).toString(),
        DS_MERCHANT_PAYMETHODS: 'z'
    };

    return obj;
}

/**
 * Get Required configuration values to pay with Redsys using credit card
 * @param {string} OrderID - Customer's order ID
 * @param {string} ammount - Amount to pay
 * @returns {Object} - Required config to pay with credit card
 */
function getRedsysCreditCardConfig(OrderID, ammount) {
    var URLUtils = require('dw/web/URLUtils');
    var Currency = require('dw/util/Currency');
    var currencies = require('*/cartridge/config/currencies');

    var obj = {};
    var merchantCurrency = null;
    // eslint-disable-next-line no-undef
    const currencyCode = session.currency.currencyCode;
    var currencyObj = !currencyCode
    ? currencies[0]
    : currencies.filter(function (currency) {
        return currency.currencyCode === currencyCode;
    })[0];

    if (!currencyObj) {
        obj = {
            error: true,
            message: 'currency error'
        };

        return obj;
    }

    obj = {
        DS_MERCHANT_AMOUNT: ammount * 100,
        DS_MERCHANT_CURRENCY: currencyObj.currencyNumber,
        DS_MERCHANT_MERCHANTCODE: redsysCreditCardPreferences.merchantCode,
        DS_MERCHANT_MERCHANTURL: URLUtils.https('Redsys-Notify').toString(),
        DS_MERCHANT_ORDER: OrderID,
        DS_MERCHANT_TERMINAL: redsysCreditCardPreferences.merchantTerminal,
        DS_MERCHANT_TRANSACTIONTYPE: '0',
        DS_MERCHANT_URLKO: URLUtils.https('Redsys-KO', 'orderID', OrderID).toString(),
        DS_MERCHANT_URLOK: URLUtils.https('Redsys-OK', 'orderID', OrderID).toString()
    };

    return obj;
}

/**
 * Encrypt message using 3DES encryption
 * @param {string} message - message to be encrypted, usually orderId is sent
 * @param {Object} key - key to be used on the 3DES encryption, this is a key provided by Redsys
 * @returns {string} Encrypted code
 */
function encrypt3DES(message, key) {
    var Bytes = require('dw/util/Bytes');
    var Cipher = require('dw/crypto/WeakCipher');
    var Encoding = require('dw/crypto/Encoding');

    // We calculate the next multiple of 8 of the message length
    var length = Math.ceil(message.length / 8) * 8;
    // We extend the message to the following multiple of 8 with null characters
    var messageExtended = message + repeatCharacter('\0', length - message.length);
    // We use 8 bytes of null characters
    var iv = new Bytes('\0\0\0\0\0\0\0\0');
    // We create a new byte and encode to base64
    var ivEncoded64 = Encoding.toBase64(iv); // AAAAAAAAAAA=
    // We encrypt it using 3DES (Triple DES) Encryption
    var CipherObj = new Cipher;
    var encrypted = CipherObj.encrypt(messageExtended, key, 'DESede/CBC/NOPADDING', ivEncoded64, 1);
    // We decode the 3DES encrypted value, because SFCC returns a base64 encoded value
    var encryptedDecoded = Encoding.fromBase64(encrypted);

    return encryptedDecoded;
}

/**
 * Create merchant signature
 * @param {string} key - Key to be used on the 3DES encryption, this is a key provided by Redsys
 * @param {Object} data - All the purchase data such as order price, order id...
 * @param {Object} order - Customer's order ID
 * @returns {string} Encoded data
 */
function createMerchantSignatureRequest(key, data, order) {
    var Bytes = require('dw/util/Bytes');
    var Encoding = require('dw/crypto/Encoding');
    var MAC = require('dw/crypto/Mac');

    // We generate Ds_MerchantParameters
    var dataJSON = JSON.stringify(data);
    var merchantParameters = Encoding.toBase64(new Bytes(dataJSON));

    // We diversify order ID with the key
    var diversifiedKey = encrypt3DES(order, key);
    // MAC256 of Ds_MerchantParameters together with the key
    var MAC256 = new MAC(MAC.HMAC_SHA_256);
    var res = MAC256.digest(merchantParameters, diversifiedKey);
    // Code data to base64
    var encodedRes = Encoding.toBase64(res);

    return encodedRes;
}

/**
 * Make payment parameters
 * @param {string} paymentMethod - Payment method
 * @param {Object} paramsInput - Paramns needed
 * @param {*} OrderId - Customer's order ID
 * @returns {Object} parameters
 */
function makeParameters(paymentMethod, paramsInput, OrderId) {
    var StringUtils = require('dw/util/StringUtils');
    var merchantSecret;
    var params = paramsInput;

    if (paymentMethod === 'REDSYS_CREDIT_CARD') {
        merchantSecret = redsysCreditCardPreferences.merchantSecret;
    }
    if (paymentMethod === 'REDSYS_BIZUM') {
        merchantSecret = redsysBizumPreferences.merchantSecret;
    }

    var DsSignature = createMerchantSignatureRequest(merchantSecret, params, OrderId);
    var parameters = {
        Ds_SignatureVersion: RedsysConstants.RESDSYS_DS_SIGNATURE_VERSION,
        Ds_MerchantParameters: StringUtils.encodeBase64(JSON.stringify(params)),
        Ds_Signature: DsSignature
    };
    return parameters;
}

module.exports = {
    getBizumConfig: getBizumConfig,
    isRedsysCreditCardEnabled: isRedsysCreditCardEnabled,
    getRedsysBizumPreferences: getRedsysBizumPreferences,
    getRedsysCreditCardPreferences: getRedsysCreditCardPreferences,
    getRedsysCreditCardConfig: getRedsysCreditCardConfig,
    isRedsysBizumEnabled: isRedsysBizumEnabled,
    getRedsysBizumRedirectionUrl: getRedsysBizumRedirectionUrl,
    getRedsysCreditCardRedirectionUrl: getRedsysCreditCardRedirectionUrl,
    makeParameters: makeParameters
};
