'use strict';

var collections = require('*/cartridge/scripts/util/collections');
var Transaction = require('dw/system/Transaction');
var RedsysConstants = require('*/cartridge/scripts/constants/redsysConstants');
var Logger = require('dw/system/Logger');
var logger = Logger.getLogger('Bizum', 'CreditCard');

/**
 * Verifies that entered Redsys Credict Card information is valid. If the information is valid a
 * Redsys Credit Card payment instrument is created
 * @param {dw.order.Basket} basket Current users's basket
 * @param {Object} paymentInformation - the payment information
 * @param {string} paymentMethodID - paymentmethodID
 * @param {Object} req the request object
 * @return {Object} returns an error object
 */
function Handle(basket, paymentInformation, paymentMethodID, req) {
    var currentBasket = basket;
    var amount = basket.totalGrossPrice;

    try {
        Transaction.wrap(function () {
            var paymentInstruments = currentBasket.getPaymentInstruments();

            collections.forEach(paymentInstruments, function (item) {
                currentBasket.removePaymentInstrument(item);
            });

            currentBasket.createPaymentInstrument(
                RedsysConstants.REDSYS_CREDIT_CARD, amount
            );
        });
    } catch (e) {
        logger.error('Error creating payment instrument: {0}', RedsysConstants.REDSYS_CREDIT_CARD);
    }

    return { fieldErrors: {}, serverErrors: [], error: false };
}


/**
 * Authorizes a payment using Redsys Bizum.
 * @param {number} orderNumber - The current order's number
 * @param {dw.order.PaymentInstrument} paymentInstrument -  The payment instrument to authorize
 * @param {dw.order.PaymentProcessor} paymentProcessor -  The payment processor of the current
 *      payment method
 * @return {Object} returns an error object
 */
function Authorize(orderNumber, paymentInstrument, paymentProcessor) {
    var Resource = require('dw/web/Resource');
    var serverErrors = [];
    var fieldErrors = {};
    var error = false;

    try {
        Transaction.wrap(function () {
            paymentInstrument.paymentTransaction.setTransactionID(orderNumber);
            paymentInstrument.paymentTransaction.setPaymentProcessor(paymentProcessor);
        });
    } catch (e) {
        error = true;
        serverErrors.push(
            Resource.msg('error.technical', 'checkout', null)
        );
    }

    return { fieldErrors: fieldErrors, serverErrors: serverErrors, error: error };
 }

exports.Handle = Handle;
exports.Authorize = Authorize;
